
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Read data from DS18B20 sensor and present it with a web page    +
//+ Based on a Script by Bernhard Linz ( https://znil.net/ )        +
//+                                                                 +
//+ by jepe, 22.12.2020                                             +
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

   // import used libraries
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library for ST7735
#include <Fonts/FreeMono9pt7b.h>
#include <Fonts/FreeMono12pt7b.h>
#include <Fonts/FreeMonoBold9pt7b.h>
#include <Fonts/FreeMonoBold12pt7b.h>
#include <Fonts/FreeMonoBold18pt7b.h>
#include <Fonts/FreeSansBold18pt7b.h>
#include <SPI.h>
#include "credentials.h"                // include WiFi credentials

    // GPIO where the DS18B20 is connected to
#define ONE_WIRE_BUS 5                  // Sensor DS18B20 am digitalen Pin D1 = GPIO5
OneWire oneWire(ONE_WIRE_BUS);          // Setup a oneWire instance to communicate with any OneWire devices
DallasTemperature sensors(&oneWire);    // Übergabe der OnewWire Referenz zum kommunizieren mit dem Sensor.

    //Anschlüsse am WEMOS D1
#define TFT_CS         15
#define TFT_RST        16                                            
#define TFT_DC         2

Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_RST);
ESP8266WebServer webServer(80);         // create web server
float temperature;                      // variables for measured values
float temperatureC;                     // 
float previoustemp;                     // letzte Temperatur
int inpval;                             // Netzzustand
int previousintval;                     // letzter Netzzustand
unsigned long vorherigeMillis = 0;      // letzte Zeit für Anzeige
unsigned long previousMillis = 0;       // letzte Zeit für email
const long interval = 3000;             // Messintervall
const long tagesinterval = 86400000;    // Tagesintervall
const int Power = 4;                    // D2 = GPIO4
char Zustand[2000];
char Server_laeuft[100];

/***********************************************************/
void setup()                            // Initialisierung
{ delay(1000);
  pinMode ( Power, INPUT );
  Serial.begin(115200);
  Serial.println("Gefrierschranküberwachung");

  tft.initR(INITR_BLACKTAB);      // Init ST7735S chip, black tab
  Serial.println(F("TFT Initialisiert"));

  uint16_t time = millis();
  time = millis() - time;
  
  tft.setRotation(1);               // dreht die Anzeige auf Landscape 
  tft.fillScreen(ST77XX_BLACK);     // löscht Display (füllt mit schwarz)
  tft.setFont(&FreeMonoBold9pt7b);  // setzt Font
  tft.setCursor(1, 20);
  tft.setTextColor(ST77XX_WHITE);    
  tft.println("Connecting to ");
  tft.println(ssid);

  WiFi.begin(ssid, password);             // connecting to local wi-fi network
  tft.setFont(&FreeMono9pt7b);
  while (WiFi.status() != WL_CONNECTED)   // check wi-fi staus until connected
  { delay(1000);
    tft.print(".");
  } 
  tft.println("");
  tft.println("WiFi connected");
  tft.print("IP: ");
  tft.println(WiFi.localIP());  
  delay(2000);
  tft.fillScreen(ST77XX_BLACK); 
  tft.setFont(&FreeMonoBold9pt7b);
  
//  uint8_t macAddr[6];

  webServer.on("/", handle_OnConnect);
  webServer.onNotFound(handle_NotFound);

  webServer.begin();
  Serial.println("HTTP server started");
  
  sensors.begin();                        // Initialize the DS18B20 temperature sensor
  Netzzustand();                          // erster Durchgang
}

//============================================================================
void loop() 
{ unsigned long currentMillis = millis();
  if (currentMillis - vorherigeMillis >= interval) // Intervall abgelaufen?
  { vorherigeMillis = currentMillis;               // ja, Zeit neu setzen
    Temperaturanzeige();                           // Aktion ausführen
  }
/*   
  if (currentMillis - previousMillis >= tagesinterval) // Intervall abgelaufen?
  { previousMillis = currentMillis;               // ja, Zeit neu setzen
    email_senden;                                 // Aktion ausführen
  }
*/
  inpval = digitalRead(Power);
  if (inpval != previousintval)                   // Netzzustand geändert?
  { Netzzustand();
 //   email_senden(); 
  }
         
  webServer.handleClient();
}

//=========================================================================

void Temperaturanzeige()
{ sensors.requestTemperatures();                    // Before actually getting the temperature, you need to call the requestTemperatures() method.
  float temperatureC = sensors.getTempCByIndex(0);  // Then, get the temperature in Celsius by using the getTempCByIndex() method as shown below:
  if (temperatureC != previoustemp)                 // neue Ausgabe nur wenn Wert geändert
  { Serial.print(temperatureC);                     // Ausgabe auf seriellem Monitor.
    Serial.println("ºC"); 

    tft.setFont(&FreeMonoBold9pt7b);                  // Titel, erste Zeile
    tft.setCursor(5, 20);
    tft.setTextColor(ST77XX_WHITE);
    tft.println("Gefrierschrank");

    if (temperatureC < -15)
    { tft.fillRect(1,40,160,40,ST77XX_GREEN);         // löscht Temperaturbereich
      tft.setCursor(1, 65);
      tft.setFont(&FreeMonoBold12pt7b);
      tft.setTextColor(ST77XX_BLACK);
      tft.setTextWrap(false); 
      tft.print(temperatureC);                        // Temperaturanzeige, 2te Zeile
      tft.println(" Grad");      
    }
    else
    { tft.fillRect(1,40,160,40,ST77XX_RED);           // löscht Temperaturbereich
      tft.setCursor(1, 65);
      tft.setFont(&FreeMonoBold12pt7b);
      tft.setTextColor(ST77XX_WHITE);
      tft.setTextWrap(false);
      tft.print(temperatureC);                        // Temperaturanzeige, 2te Zeile
      tft.println(" Grad");
    }
  }
  previoustemp = temperatureC; 
}

void Netzzustand()
{ if (inpval == HIGH)
  { tft.fillRect(1,80,160,47,ST77XX_GREEN);         // löscht Spannungsbereich
    tft.setFont(&FreeMonoBold9pt7b);
    tft.setCursor(1, 100);
    tft.setTextColor(ST77XX_BLACK);
    tft.println(" 230V-Speisung");                  // Netzzustand, 3te Zeile
    tft.println("  in Ordnung");       
    strcpy(Zustand , "<tr><td bgcolor=green><p><center><h3><b><font color=white>230V in Ordnung</font></b></h3></center></p></td></tr>");    
  }
  else
  { tft.fillRect(1,80,160,47,ST77XX_RED);           // löscht Spannungsbereich
    tft.setFont(&FreeMonoBold9pt7b);
    tft.setCursor(1, 100);
    tft.setTextColor(ST77XX_WHITE);
    tft.println(" 230V-Speisung");    
    tft.println("    AUSFALL");       
    strcpy(Zustand , "<tr><td bgcolor=red><p><center><h3><b><font color=white>230V AUSFALL</font></b></h3></center></p></td></tr>");    
  }
  previousintval = inpval;    
}

void handle_OnConnect() 

{ int sec = millis() / 1000;
  int min = sec / 60;
  int hr = min / 60;
  snprintf ( Server_laeuft , 100 , "<h3>Server l&aumluft: %02d:%02d:%02d</font></h3>" , hr, min % 60, sec % 60);

  sensors.requestTemperatures();                    // Before actually getting the temperature, you need to call the requestTemperatures() method.
  float temperatureC = sensors.getTempCByIndex(0);  // Then, get the temperature in Celsius by using the getTempCByIndex() method as shown below:
  if (temperatureC < -15)
  { webServer.send(200, "text/html", buildHtml_green(temperatureC));
    webServer.send(200, "text/html", Server_laeuft);  
  }
  else
  { webServer.send(200, "text/html", buildHtml_red(temperatureC));
    webServer.send(200, "text/html", Server_laeuft);    
  }   
}

void handle_NotFound()
{ webServer.send(404, "text/plain", "Not found");
}

String buildHtml_red(float _temperatureC)
{ String page = "<!DOCTYPE html> <html>\n";
  page +="<head><meta name='viewport' content='width=device-width, initial-scale=1.0, user-scalable=no'/>\n";
  page +="<meta charset='UTF-8'/>";
  page +="<meta http-equiv='refresh' content='5'/>";
  page +="<title>Temperaturueberwachung</title>\n";
  page +="<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n";
  page +="body{margin-top: 50px;} h1 {color: #444444;margin: 20px auto 30px;}\n";
  page +="h2 {color: #0d4c75;margin: 50px auto 20px;}\n";
  page +="p {font-size: 24px;color: #444444;margin-bottom: 10px;}\n";
  page +="</style>\n";
  page +="</head>\n";  
  page +="<body>\n";
  page +="<div id='webpage'>\n";
  page +="<h2>Gefrierschrank&uumlberwachung</h2>\n"; 
  page += Server_laeuft;
  page +="<center><table border bgcolor=black><tr><td bgcolor=red><p><h3><font color=white>Gefrierschranktemperatur: ";
  page +=(float)_temperatureC;
  page +=" °C";
  page +=Zustand;
  page +="</font></h3></p></td></tr></table></center>";
  page +="</div>\n";
  page +="</body>\n";
  page +="</html>\n";
  return page; 
}

String buildHtml_green(float _temperatureC)
{ String page = "<!DOCTYPE html> <html>\n";
  page +="<head><meta name='viewport' content='width=device-width, initial-scale=1.0, user-scalable=no'/>\n";
  page +="<meta charset='UTF-8'/>";
  page +="<meta http-equiv='refresh' content='5'/>";
  page +="<title>Temperaturueberwachung</title>\n";
  page +="<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n";
  page +="body{margin-top: 50px;} h1 {color: #444444;margin: 20px auto 30px;}\n";
  page +="h2 {color: #0d4c75;margin: 50px auto 20px;}\n";
  page +="p {font-size: 24px;color: #444444;margin-bottom: 10px;}\n";
  page +="</style>\n";
  page +="</head>\n";  
  page +="<body>\n";
  page +="<div id='webpage'>\n";
  page +="<h2>Gefrierschrank&uumlberwachung</h2>\n"; 
  page +=Server_laeuft;
  page +="<center><table border bgcolor=black><tr><td bgcolor=green><p><h3><font color=white>Gefrierschranktemperatur: ";
  page +=(float)_temperatureC;
  page +=" °C";
  page +=Zustand;
  page +="</font></h3></p></td></tr></table></center>";
  page +="</div>\n";
  page +="</body>\n";
  page +="</html>\n";
  return page; 
}


//******* Text senden falls Bedingung erfüllt ist *******
void email_senden()
{ if (inpval  == true)
//  { sendmails(email_green(temperatureC));
//  Serial.printf ("Speisung vorhanden    %f Grad C", temperatureC);
 
  { sendmails("Speisung vorhanden");
  Serial.println ("Speisung vorhanden");
  }
  else
  { sendmails("Speisung unterbrochen!!! ");
  Serial.println ("Speisung unterbrochen!!!");
  }
}  


String email_green(float _temperatureC)
{ String page = "Speisung vorhanden,   ";
  page += (float)_temperatureC;
  page += " Grad C";
  return page;
}

  
